@@ -97,16 +97,23 @@ sub digestNextBody {
   my @aug       = ();
 
   while (defined($token = $$self{gullet}->readXToken(1, 1))) {    # Done if we run out of tokens
+    if ($alignment && scalar(@LaTeXML::LIST) && (Equals($token, T_ALIGN) ||
+        Equals($token, T_CS('\cr')) || Equals($token, T_CS('\hidden@cr')) ||
+        Equals($token, T_CS('\hidden@crcr')))) {
+      # at least \over calls in here without the intent to passing through the alignment.
+      # So if we already have some digested boxes available, return them here.
+      $$self{gullet}->unread($token);
+      return @LaTeXML::LIST; }
     my @r = $self->invokeToken($token);
     push(@LaTeXML::LIST, @r);
     push(@aug, $token, @r);
     last if $terminal and Equals($token, $terminal);
-    last if $initdepth > scalar(@{ $$self{boxing} }); }           # if we've closed the initial mode.
+    last if $initdepth > scalar(@{ $$self{boxing} }); }    # if we've closed the initial mode.
   Warn('expected', $terminal, $self, "body should have ended with '" . ToString($terminal) . "'",
     "current body started at " . ToString($startloc),
     "Got " . join("\n -- ", map { Stringify($_) } @aug))
     if $terminal && !Equals($token, $terminal);
-  push(@LaTeXML::LIST, Box()) unless $token;                      # Dummy `trailer' if none explicit.
+  push(@LaTeXML::LIST, Box()) unless $token;               # Dummy `trailer' if none explicit.
   return @LaTeXML::LIST; }
 
 # Digest a list of tokens independent from any current Gullet.
@@ -146,8 +153,8 @@ our @CATCODE_ABSORBABLE = (    # [CONSTANT]
   0, 0, 0, 0);
 
 sub invokeToken {
-  my ($self, $token) = @_;
   no warnings 'recursion';
+  my ($self, $token) = @_;
 INVOKE:
   ProgressStep() if ($$self{progress}++ % $DIGESTION_PROGRESS_QUANTUM) == 0;
   push(@{ $$self{token_stack} }, $token);
