@@ -35,7 +35,7 @@ sub convertNode_linebreak {
     require LaTeXML::Post::MathML::Linebreaker;
     $breaker = $$self{linebreaker} = LaTeXML::Post::MathML::Linebreaker->new(); }
 
-  my $pmml = $self->convertNode_simple($doc, $xmath, $style);
+  my $pmml   = $self->convertNode_simple($doc, $xmath, $style);
   my $layout = $breaker->bestFitToWidth($xmath, $pmml, $$self{linelength}, 1);
   if ($$layout{hasbreak}) {    # YES it did linebreak!
     $pmml = $breaker->applyLayout($pmml, $layout); }
@@ -57,7 +57,7 @@ sub convertNode {
   # NEXT better strategy will be to scan columns of MathBranches to establish desired line length?
   elsif ($$self{linelength}    # If line breaking
     && ($doc->findnodes('ancestor::ltx:MathBranch', $xmath))    # In formatted side of MathFork?
-          # But ONLY if last column!! (until we can adapt LineBreaker!)
+        # But ONLY if last column!! (until we can adapt LineBreaker!)
     && !$doc->findnodes('parent::ltx:Math/parent::ltx:td/following-sibling::ltx:td', $xmath)) {
     my ($pmmlb, $broke) = $self->convertNode_linebreak($doc, $xmath, $style);
     $pmml = $pmmlb; }
@@ -68,6 +68,24 @@ sub convertNode {
 sub rawIDSuffix {
   return '.pmml'; }
 
+sub associateNodeHook {
+  my ($self, $node, $sourcenode) = @_;
+  # TODO: Shouldn't we have a single getQName shared for the entire latexml codebase
+  #  in LaTeXML::Common or LaTeXML::Util ?
+  my $name = LaTeXML::Post::MathML::getQName($node);
+  if ($name =~ /^m:(?:mi|mo|mn)$/) {
+    if (my $href = $sourcenode->getAttribute('href')) {
+      if (ref $node eq 'ARRAY') {
+        $$node[1]{href} = $href; }
+      else {
+        $node->setAttribute('href', $href); } }
+    if (my $title = $sourcenode->getAttribute('title')) {
+      if (ref $node eq 'ARRAY') {
+        $$node[1]{title} = $title; }
+      else {
+        $node->setAttribute('title', $title); } } }
+  return; }
+
 #================================================================================
 # Presentation MathML with Line breaking
 # Not at all sure how this will integrate with Parallel markup...
@@ -98,9 +116,9 @@ sub preprocess_linebreaking {
     my $style = ($mode eq 'display' ? 'display' : 'text');
     # If already has in a MathBranch, we can't really know if, or how wide, to line break!?!?!
     next if $doc->findnodes('ancestor::ltx:MathFork', $math);    # SKIP if already in a branch?
-          # Now let's do the layout & see if it actually needs line breaks!
-          # next if $math isn't really so wide ..
-    my $id = $math->getAttribute('xml:id');
+        # Now let's do the layout & see if it actually needs line breaks!
+        # next if $math isn't really so wide ..
+    my $id    = $math->getAttribute('xml:id');
     my $xmath = $doc->findnode('ltx:XMath', $math);
     my ($pmml, $broke) = $self->convertNode_linebreak($doc, $xmath, $style);
     if ($broke) {    # YES it did linebreak!
